package Courier::Filter::Module::Greylist;
use base qw(Courier::Filter::Module);
use Socket;
use Net::IP qw(:PROC);

sub new {
    my ($class, %options) = @_;
    my $conffile = "/etc/greylistd/config";
    my %config = ();

    if (-f $conffile) {
	if (open(GREYCONF,$conffile)) {
	    my $section = "DEFAULT";
	    my $linecnt = 0;
	    while(<GREYCONF>) {
		$linecnt++;
		chomp;
		print STDERR "DEBUG: <config[$linecnt]: $_\n"
		    if $options{debugging};
		next if (/^\#/ || /^\s*$/);	# Skip comments and blank lines
		if (/^\s*\[([^\]]+)\]\s*(\#.*)?$/) {
		    $section = $1;
		} elsif (/^\s*([^\s]+)\s*[:=]\s*(.*)$/) {
		    my $key = $1;
		    my $value = $2;
		    if (!defined $config{$section}) {
			$config{$section} = {};
		    }
		    print STDERR "DEBUG: \$config{$section}->{$key} = $value\n"
			if $options{debugging};
		    $config{$section}->{$key} = $value;
		} else {
		    die "Syntax error in configuration file $conffile:\n\"$_\"\n";
		}
	    }
	    close(GREYCONF);
	} else {
	    die "Reading configuration file $conffile: $!\n";
	}
    }

    my $sockfile = $config{socket}->{path};
    $sockfile = "/var/run/greylistd/socket" if (!defined $sockfile);

    my %new_options = ();
    if (defined $options{netmask_v4}) {
	$new_options{netmask_v4} =
	    "1" x $options{netmask_v4} . "0" x (32 - $options{netmask_v4});
    }
    if (defined $options{netmask_v6}) {
	$new_options{netmask_v6} =
	    "1" x $options{netmask_v6} . "0" x (128 - $options{netmask_v6});
    }
    if (defined $options{white_networks}) {
	$new_options{white_networks} =
	    [ map { new Net::IP($_) } @{$options{white_networks}} ];
    }
    if (defined $options{debug}) {
	$new_options{debug} = $options{debug};
    } else {
	$new_options{debug} = 0;
    }
    my $module = $class->SUPER::new( %new_options,
				     grey_sock_file => $sockfile );
    return $module;
}

my %exits = ( "grey"	=>	[ "4.7.1 Temporarily rejected", 451 ],
	      "black"	=>	[ "5.7.1 Rejected", 550 ] );

sub match {
    my ($module, $message) = @_;

    socket(GREYSOCK, PF_UNIX, SOCK_STREAM, 0)
	|| die "socket: $!\n";
    connect(GREYSOCK, sockaddr_un($module->{grey_sock_file}))
	|| die "connect to " . $module->{grey_sock_file} . ": $!\n";

    my $addr = new Net::IP($message->remote_host);

    if ($addr->version() == 4 && defined $module->{netmask_v4}) {
	if ($module->{debug}) {
	    my $tmp_mask = new Net::IP(ip_bintoip($module->{netmask_v4},4));
	    print STDERR "DEBUG: ",$addr->short()," masked with ",$tmp_mask->short(),": ";
	}
	$addr = new Net::IP(ip_bintoip($addr->binip() & $module->{netmask_v4},
				       4));
	print STDERR $addr->short(),"\n"
	    if $module->{debug};
    } elsif ($addr->version() == 6 && defined $module->{netmask_v6}) {
	if ($module->{debug}) {
	    my $tmp_mask = new Net::IP(ip_bintoip($module->{netmask_v6},6));
	    print STDERR "DEBUG: ",$addr->short()," masked with ",$tmp_mask->short(),": ";
	}
	$addr = new Net::IP(ip_bintoip($addr->binip() & $module->{netmask_v6},
				       6));
	print STDERR $addr->short(),"\n"
	    if $module->{debug};
    }

    if (defined $module->{white_networks}) {
	foreach my $network (@{$module->{white_networks}}) {
	    if ($addr->version() == $network->version()) {
		my $res = $addr->overlaps($network);
		print STDERR "DEBUG: testing if ",$addr->short()," is in network ",$network->short(),"/",$network->prefixlen(),": "
		    if $module->{debug};
		if ($res == $IP_IDENTICAL || $res == $IP_A_IN_B_OVERLAP) {
		    print STDERR "yes\n";
		    return undef;
		}
		print STDERR "no\n"
		    if $module->{debug};
	    }
	}
    }

    my $mess = "update "
	.$addr->short()." "
	.$message->sender." "
	.$message->recipients;
    send GREYSOCK,$mess,0;
    print STDERR "DEBUG: >$mess\n"
	if $module->{debug};
    my $reply = "";
    recv GREYSOCK,$reply,1024,0;
    print STDERR "DEBUG: <$reply\n"
	if $module->{debug};

    close(GREYSOCK);

    $firstword = (split(/\s+/,$reply))[0];
    $retval = $exits{$firstword};
    return @$retval if (defined $retval);
    return undef;
}
